@@ -36,7 +36,8 @@
 #'
 #' @rdname FileInfo
 #' @export
-FileInfo <- R6Class("FileInfo",
+FileInfo <- R6Class(
+  "FileInfo",
   inherit = ArrowObject,
   public = list(
     base_name = function() fs___FileInfo__base_name(self),
@@ -96,7 +97,8 @@ FileInfo <- R6Class("FileInfo",
 #'
 #' @rdname FileSelector
 #' @export
-FileSelector <- R6Class("FileSelector",
+FileSelector <- R6Class(
+  "FileSelector",
   inherit = ArrowObject,
   active = list(
     base_dir = function() fs___FileSelector__base_dir(self),
@@ -156,6 +158,10 @@ FileSelector$create <- function(base_dir, allow_not_found = FALSE, recursive = F
 #'    buckets if `$CreateDir()` is called on the bucket level (default `FALSE`).
 #' - `allow_bucket_deletion`: logical, if TRUE, the filesystem will delete
 #'    buckets if`$DeleteDir()` is called on the bucket level (default `FALSE`).
+#' - `check_directory_existence_before_creation`: logical, check if directory
+#'    already exists or not before creation. Helpful for cloud storage operations
+#'    where object mutation operations are rate limited or existing directories
+#'    are read-only. (default `FALSE`).
 #' - `request_timeout`: Socket read time on Windows and macOS in seconds. If
 #'    negative, the AWS SDK default (typically 3 seconds).
 #' - `connect_timeout`: Socket connection timeout in seconds. If negative, AWS
@@ -254,7 +260,8 @@ FileSelector$create <- function(base_dir, allow_not_found = FALSE, recursive = F
 #' @rdname FileSystem
 #' @name FileSystem
 #' @export
-FileSystem <- R6Class("FileSystem",
+FileSystem <- R6Class(
+  "FileSystem",
   inherit = ArrowObject,
   public = list(
     GetFileInfo = function(x) {
@@ -390,7 +397,8 @@ are_urls <- function(x) if (!is.character(x)) FALSE else grepl("://", x)
 #' @export
 LocalFileSystem <- R6Class("LocalFileSystem", inherit = FileSystem)
 LocalFileSystem$create <- function() {
-  fs___LocalFileSystem__create()
+  # from_uri needs a non-empty path, so just use a placeholder of /_
+  FileSystem$from_uri("file:///_")$fs
 }
 
 #' @usage NULL
@@ -398,7 +406,8 @@ LocalFileSystem$create <- function() {
 #' @rdname FileSystem
 #' @importFrom utils modifyList
 #' @export
-S3FileSystem <- R6Class("S3FileSystem",
+S3FileSystem <- R6Class(
+  "S3FileSystem",
   inherit = FileSystem,
   active = list(
     region = function() fs___S3FileSystem__region(self)
@@ -409,8 +418,16 @@ S3FileSystem$create <- function(anonymous = FALSE, ...) {
   if (anonymous) {
     invalid_args <- intersect(
       c(
-        "access_key", "secret_key", "session_token", "role_arn", "session_name",
-        "external_id", "load_frequency", "allow_bucket_creation", "allow_bucket_deletion"
+        "access_key",
+        "secret_key",
+        "session_token",
+        "role_arn",
+        "session_name",
+        "external_id",
+        "load_frequency",
+        "allow_bucket_creation",
+        "allow_bucket_deletion",
+        "check_directory_existence_before_creation"
       ),
       names(args)
     )
@@ -458,6 +475,7 @@ default_s3_options <- list(
   background_writes = TRUE,
   allow_bucket_creation = FALSE,
   allow_bucket_deletion = FALSE,
+  check_directory_existence_before_creation = FALSE,
   connect_timeout = -1,
   request_timeout = -1
 )
@@ -481,13 +499,13 @@ default_s3_options <- list(
 #' relative path. Note that this function's success does not guarantee that you
 #' are authorized to access the bucket's contents.
 #' @examplesIf FALSE
-#' bucket <- s3_bucket("voltrondata-labs-datasets")
+#' bucket <- s3_bucket("arrow-datasets")
 #'
 #' @examplesIf FALSE
 #' # Turn on debug logging. The following line of code should be run in a fresh
 #' # R session prior to any calls to `s3_bucket()` (or other S3 functions)
-#' Sys.setenv("ARROW_S3_LOG_LEVEL"="DEBUG")
-#' bucket <- s3_bucket("voltrondata-labs-datasets")
+#' Sys.setenv("ARROW_S3_LOG_LEVEL" = "DEBUG")
+#' bucket <- s3_bucket("arrow-datasets")
 #'
 #' @export
 s3_bucket <- function(bucket, ...) {
@@ -523,7 +541,7 @@ s3_bucket <- function(bucket, ...) {
 #' relative path. Note that this function's success does not guarantee that you
 #' are authorized to access the bucket's contents.
 #' @examplesIf FALSE
-#' bucket <- gs_bucket("voltrondata-labs-datasets")
+#' bucket <- gs_bucket("arrow-datasets")
 #' @export
 gs_bucket <- function(bucket, ...) {
   assert_that(is.string(bucket))
@@ -538,7 +556,8 @@ gs_bucket <- function(bucket, ...) {
 #' @format NULL
 #' @rdname FileSystem
 #' @export
-GcsFileSystem <- R6Class("GcsFileSystem",
+GcsFileSystem <- R6Class(
+  "GcsFileSystem",
   inherit = FileSystem,
   active = list(
     options = function() {
@@ -547,7 +566,9 @@ GcsFileSystem <- R6Class("GcsFileSystem",
       # Convert from nanoseconds to POSIXct w/ UTC tz
       if ("expiration" %in% names(out)) {
         out$expiration <- as.POSIXct(
-          out$expiration / 1000000000, origin = "1970-01-01", tz = "UTC"
+          out$expiration / 1000000000,
+          origin = "1970-01-01",
+          tz = "UTC"
         )
       }
 
@@ -584,8 +605,14 @@ GcsFileSystem$create <- function(anonymous = FALSE, retry_limit_seconds = 15, ..
   }
 
   valid_opts <- c(
-    "access_token", "expiration", "json_credentials", "endpoint_override",
-    "scheme", "default_bucket_location", "default_metadata", "project_id"
+    "access_token",
+    "expiration",
+    "json_credentials",
+    "endpoint_override",
+    "scheme",
+    "default_bucket_location",
+    "default_metadata",
+    "project_id"
   )
 
   invalid_opts <- setdiff(names(options), valid_opts)
@@ -602,8 +629,10 @@ GcsFileSystem$create <- function(anonymous = FALSE, retry_limit_seconds = 15, ..
     stop(
       paste(
         "Option 'expiration' must be of class POSIXct, not",
-        class(options$expiration)[[1]]),
-      call. = FALSE)
+        class(options$expiration)[[1]]
+      ),
+      call. = FALSE
+    )
   }
 
   options$retry_limit_seconds <- retry_limit_seconds
@@ -620,13 +649,17 @@ GcsFileSystem$create <- function(anonymous = FALSE, retry_limit_seconds = 15, ..
 #' @format NULL
 #' @rdname FileSystem
 #' @export
-SubTreeFileSystem <- R6Class("SubTreeFileSystem",
+SubTreeFileSystem <- R6Class(
+  "SubTreeFileSystem",
   inherit = FileSystem,
   public = list(
     print = function(...) {
       cat(
         "SubTreeFileSystem: ",
-        self$url_scheme, "://", self$base_path, "\n",
+        self$url_scheme,
+        "://",
+        self$base_path,
+        "\n",
         sep = ""
       )
       invisible(self)
