@@ -151,8 +151,8 @@ static inline bool can_convert_native(SEXP x) {
     return Rf_inherits(x, "factor") || Rf_inherits(x, "Date") ||
            Rf_inherits(x, "integer64") || Rf_inherits(x, "POSIXct") ||
            Rf_inherits(x, "hms") || Rf_inherits(x, "difftime") ||
-           Rf_inherits(x, "data.frame") || Rf_inherits(x, "arrow_binary") ||
-           Rf_inherits(x, "arrow_large_binary") ||
+           Rf_inherits(x, "data.frame") || Rf_inherits(x, "blob") ||
+           Rf_inherits(x, "arrow_binary") || Rf_inherits(x, "arrow_large_binary") ||
            Rf_inherits(x, "arrow_fixed_size_binary") ||
            Rf_inherits(x, "vctrs_unspecified") || Rf_inherits(x, "AsIs");
   }
@@ -246,9 +246,7 @@ arrow::Status AddMetadataFromDots(SEXP lst, int num_fields,
 
 namespace altrep {
 
-#if defined(HAS_ALTREP)
 void Init_Altrep_classes(DllInfo* dll);
-#endif
 
 SEXP MakeAltrepVector(const std::shared_ptr<ChunkedArray>& chunked_array);
 bool is_arrow_altrep(SEXP x);
