@@ -35,9 +35,11 @@ exit <- function(..., .status = 1) {
 
 
 # checks the nightly repo for the latest nightly version X.Y.Z.100<dev>
-find_latest_nightly <- function(description_version,
-                                list_uri = "https://nightlies.apache.org/arrow/r/src/contrib/PACKAGES",
-                                hush = quietly) {
+find_latest_nightly <- function(
+  description_version,
+  list_uri = "https://nightlies.apache.org/arrow/r/src/contrib/PACKAGES",
+  hush = quietly
+) {
   if (!startsWith(arrow_repo, "https://nightlies.apache.org/arrow/r")) {
     lg("Detected non standard dev repo: %s, not checking latest nightly version.", arrow_repo)
     return(description_version)
@@ -107,8 +109,7 @@ validate_checksum <- function(binary_url, libfile, hush = quietly) {
   enforce_checksum <- env_is("ARROW_R_ENFORCE_CHECKSUM", "true")
   checksum_path <- Sys.getenv("ARROW_R_CHECKSUM_PATH", "tools/checksums")
   # validate binary checksum for CRAN release only
-  if (!skip_checksum && dir.exists(checksum_path) && is_release ||
-    enforce_checksum) {
+  if (!skip_checksum && dir.exists(checksum_path) && is_release || enforce_checksum) {
     # Munge the path to the correct sha file which we include during the
     # release process
     if (VERSION_22_OR_LATER) {
@@ -128,7 +129,8 @@ validate_checksum <- function(binary_url, libfile, hush = quietly) {
       args = c("--status", "-a", "512", "-c", checksum_file),
       stdout = ifelse(quietly, FALSE, ""),
       stderr = ifelse(quietly, FALSE, "")
-    )) == 0
+    )) ==
+      0
 
     if (!checksum_ok) {
       checksum_ok <- suppressWarnings(system2(
@@ -136,7 +138,8 @@ validate_checksum <- function(binary_url, libfile, hush = quietly) {
         args = c("--status", "-c", checksum_file),
         stdout = ifelse(quietly, FALSE, ""),
         stderr = ifelse(quietly, FALSE, "")
-      )) == 0
+      )) ==
+        0
     }
 
     if (checksum_ok) {
@@ -169,7 +172,8 @@ download_binary <- function(lib) {
     # TODO: should we condense these together and only call them when verbose?
     lg(
       "Unable to retrieve libarrow for version %s (%s)",
-      VERSION, lib
+      VERSION,
+      lib
     )
     if (!quietly) {
       lg(
@@ -188,17 +192,12 @@ download_binary <- function(lib) {
 # of action based on the current system. Other values you can set it to:
 # * "FALSE" (not case-sensitive), to skip this option altogether
 # * "TRUE" (not case-sensitive), to try to discover your current OS, or
-# * Some other string: a "linux-openssl-${OPENSSL_VERSION}" that corresponds to
-#   a binary that is available, to override what this function may discover by
-#   default.
+# * Some other string: a binary identifier that corresponds to a binary that is
+#   available, to override what this function may discover by default.
 #   Possible values are:
-#    * "linux-x86_64-openssl-1.0" (OpenSSL 1.0)
-#    * "linux-x86_64-openssl-1.1" (OpenSSL 1.1)
-#    * "linux-x86_64-openssl-3.0" (OpenSSL 3.0)
-#    * "macos-arm64-openssl-1.1" (OpenSSL 1.1)
-#    * "macos-arm64-openssl-3.0" (OpenSSL 3.0)
-#    * "macos-x86_64-openssl-1.1" (OpenSSL 1.1)
-#    * "macos-x86_64-openssl-3.0" (OpenSSL 3.0)
+#    * "linux-x86_64"
+#    * "darwin-arm64"
+#    * "darwin-x86_64"
 #    * "windows-x86_64"
 #   These string values, along with `NULL`, are the potential return values of
 #   this function.
@@ -224,6 +223,17 @@ identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()
     # Env var provided an os-version to use, to override our logic.
     # We don't validate that this exists. If it doesn't, the download will fail
     # and the build will fall back to building from source
+    if (grepl("openssl-1", lib)) {
+      stop(
+        "OpenSSL 1.x binaries are no longer provided. Use LIBARROW_BINARY='",
+        sub("-openssl-1.*$", "", lib),
+        "'"
+      )
+    }
+    if (grepl("openssl-3", lib)) {
+      lib <- sub("-openssl-3.*$", "", lib)
+      lg("OpenSSL suffix deprecated in LIBARROW_BINARY, using '%s'", lib)
+    }
   } else {
     # See if we can find a suitable binary
     lib <- select_binary()
@@ -231,7 +241,10 @@ identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()
   lib
 }
 
-check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apache/arrow/main/r/tools/nixlibs-allowlist.txt") {
+check_allowlist <- function(
+  os,
+  allowed = "https://raw.githubusercontent.com/apache/arrow/main/r/tools/nixlibs-allowlist.txt"
+) {
   allowlist <- tryCatch(
     # Try a remote allowlist so that we can add/remove without a release
     suppressWarnings(readLines(allowed)),
@@ -242,9 +255,11 @@ check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apa
   any(grepl(paste(allowlist, collapse = "|"), os))
 }
 
-select_binary <- function(os = tolower(Sys.info()[["sysname"]]),
-                          arch = tolower(Sys.info()[["machine"]]),
-                          test_program = test_for_curl_and_openssl) {
+select_binary <- function(
+  os = tolower(Sys.info()[["sysname"]]),
+  arch = tolower(Sys.info()[["machine"]]),
+  test_program = test_for_curl_and_openssl
+) {
   if (identical(os, "darwin") || (identical(os, "linux") && identical(arch, "x86_64"))) {
     # We only host x86 linux binaries and x86 & arm64 macos today
     binary <- tryCatch(
@@ -252,11 +267,10 @@ select_binary <- function(os = tolower(Sys.info()[["sysname"]]),
       # so globally handle the possibility that this could fail
       {
         errs <- compile_test_program(test_program)
-        openssl_version <- determine_binary_from_stderr(errs)
-        if (is.null(openssl_version)) {
-          NULL
+        if (has_binary_sysreqs(errs)) {
+          paste0(os, "-", arch)
         } else {
-          paste0(os, "-", arch, "-", openssl_version)
+          NULL
         }
       },
       error = function(e) {
@@ -284,14 +298,8 @@ test_for_curl_and_openssl <- "
 
 #include <curl/curl.h>
 #include <openssl/opensslv.h>
-#if OPENSSL_VERSION_NUMBER < 0x10002000L
-#error OpenSSL version too old
-#endif
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-#error Using OpenSSL version 1.0
-#endif
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-#error Using OpenSSL version 3
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+#error OpenSSL version must be 3.0 or greater
 #endif
 "
 
@@ -302,11 +310,11 @@ compile_test_program <- function(code) {
     openssl_dir <- paste0("-I", openssl_root_dir, "/include")
   }
   runner <- paste(
-    R_CMD_config("CXX17"),
+    R_CMD_config("CXX20"),
     openssl_dir,
     R_CMD_config("CPPFLAGS"),
-    R_CMD_config("CXX17FLAGS"),
-    R_CMD_config("CXX17STD"),
+    R_CMD_config("CXX20FLAGS"),
+    R_CMD_config("CXX20STD"),
     "-E",
     "-xc++"
   )
@@ -330,40 +338,27 @@ get_macos_openssl_dir <- function() {
   openssl_root_dir
 }
 
-# (built with newer devtoolset but older glibc (2.17) for broader compatibility, like manylinux2014)
-determine_binary_from_stderr <- function(errs) {
-  if (is.null(attr(errs, "status"))) {
-    # There was no error in compiling: so we found libcurl and OpenSSL >= 1.1,
-    # openssl is < 3.0
-    lg("Found libcurl and OpenSSL >= 1.1")
-    return("openssl-1.1")
-    # Else, check for dealbreakers:
-  } else if (!on_macos && any(grepl("Using libc++", errs, fixed = TRUE))) {
+has_binary_sysreqs <- function(errs) {
+  # Check for dealbreakers:
+  if (!on_macos && any(grepl("Using libc++", errs, fixed = TRUE))) {
     # Our linux binaries are all built with GNU stdlib so they fail with libc++
     lg("Linux binaries incompatible with libc++")
-    return(NULL)
+    return(FALSE)
   } else if (header_not_found("curl/curl", errs)) {
     lg("libcurl not found")
-    return(NULL)
+    return(FALSE)
   } else if (header_not_found("openssl/opensslv", errs)) {
     lg("OpenSSL not found")
-    return(NULL)
-  } else if (any(grepl("OpenSSL version too old", errs))) {
-    lg("OpenSSL found but version >= 1.0.2 is required for some features")
-    return(NULL)
-    # Else, determine which other binary will work
-  } else if (any(grepl("Using OpenSSL version 1.0", errs))) {
-    if (on_macos) {
-      lg("OpenSSL 1.0 is not supported on macOS")
-      return(NULL)
-    }
-    lg("Found libcurl and OpenSSL < 1.1")
-    return("openssl-1.0")
-  } else if (any(grepl("Using OpenSSL version 3", errs))) {
-    lg("Found libcurl and OpenSSL >= 3.0.0")
-    return("openssl-3.0")
+    return(FALSE)
+  } else if (any(grepl("OpenSSL version must be 3.0 or greater", errs))) {
+    lg("OpenSSL found but version >= 3.0 is required")
+    return(FALSE)
+  } else if (is.null(attr(errs, "status"))) {
+    # Successful compile = OpenSSL >= 3.0 found
+    lg("Found libcurl and OpenSSL >= 3.0")
+    return(TRUE)
   }
-  NULL
+  FALSE
 }
 
 header_not_found <- function(header, errs) {
@@ -570,8 +565,11 @@ build_libarrow <- function(src_dir, dst_dir) {
     # is found, it will be used by the libarrow build, and this does
     # not affect how R compiles the arrow bindings.
     CC = sub("^.*ccache", "", R_CMD_config("CC")),
-    CXX = paste(sub("^.*ccache", "", R_CMD_config("CXX17")), R_CMD_config("CXX17STD")),
-    # CXXFLAGS = R_CMD_config("CXX17FLAGS"), # We don't want the same debug symbols
+    CXX = paste(
+      sub("^.*ccache", "", R_CMD_config("CXX20")),
+      R_CMD_config("CXX20STD")
+    ),
+    # CXXFLAGS = R_CMD_config("CXX20FLAGS"), # We don't want the same debug symbols
     LDFLAGS = R_CMD_config("LDFLAGS"),
     N_JOBS = ncores
   )
@@ -675,13 +673,17 @@ ensure_cmake <- function(cmake_minimum_required = "3.26") {
     } else {
       exit(paste0(
         "*** cmake was not found locally.\n",
-        "    Please make sure cmake >= ", cmake_minimum_required,
+        "    Please make sure cmake >= ",
+        cmake_minimum_required,
         " is installed and available on your PATH."
       ))
     }
     cmake_binary_url <- paste0(
-      "https://github.com/Kitware/CMake/releases/download/v", CMAKE_VERSION,
-      "/cmake-", CMAKE_VERSION, postfix
+      "https://github.com/Kitware/CMake/releases/download/v",
+      CMAKE_VERSION,
+      "/cmake-",
+      CMAKE_VERSION,
+      postfix
     )
     cmake_tar <- tempfile()
     cmake_dir <- tempfile()
@@ -689,9 +691,12 @@ ensure_cmake <- function(cmake_minimum_required = "3.26") {
     if (!download_successful) {
       exit(paste0(
         "*** cmake was not found locally and download failed.\n",
-        "    Make sure cmake >= ", cmake_minimum_required,
+        "    Make sure cmake >= ",
+        cmake_minimum_required,
         " is installed and available on your PATH,\n",
-        "    or download ", cmake_binary_url, "\n",
+        "    or download ",
+        cmake_binary_url,
+        "\n",
         "    and define the CMAKE environment variable.\n"
       ))
     }
@@ -706,7 +711,9 @@ ensure_cmake <- function(cmake_minimum_required = "3.26") {
     }
     cmake <- paste0(
       cmake_dir,
-      "/cmake-", CMAKE_VERSION, sub(".tar.gz", "", postfix, fixed = TRUE),
+      "/cmake-",
+      CMAKE_VERSION,
+      sub(".tar.gz", "", postfix, fixed = TRUE),
       "/",
       bin_dir,
       "/cmake"
@@ -716,14 +723,16 @@ ensure_cmake <- function(cmake_minimum_required = "3.26") {
   cmake
 }
 
-find_cmake <- function(paths = c(
-                         Sys.getenv("CMAKE"),
-                         Sys.which("cmake"),
-                         # CRAN has it here, not on PATH
-                         if (on_macos) "/Applications/CMake.app/Contents/bin/cmake",
-                         Sys.which("cmake3")
-                       ),
-                       version_required) {
+find_cmake <- function(
+  paths = c(
+    Sys.getenv("CMAKE"),
+    Sys.which("cmake"),
+    # CRAN has it here, not on PATH
+    if (on_macos) "/Applications/CMake.app/Contents/bin/cmake",
+    Sys.which("cmake3")
+  ),
+  version_required
+) {
   # Given a list of possible cmake paths, return the first one that exists and is new enough
   # version_required should be a string or packageVersion; numeric version
   # can be misleading (e.g. 3.10 is actually 3.1)
@@ -816,9 +825,7 @@ get_component_names <- function() {
   if (!deps_bash_success) {
     stop("Failed to run download_dependencies_R.sh")
   }
-  deps_df <- read.csv(csv_tempfile,
-    stringsAsFactors = FALSE, row.names = NULL, quote = "'"
-  )
+  deps_df <- read.csv(csv_tempfile, stringsAsFactors = FALSE, row.names = NULL, quote = "'")
   stopifnot(
     names(deps_df) == c("env_varname", "filename"),
     nrow(deps_df) > 0
@@ -912,8 +919,10 @@ cmake_find_package <- function(pkg, version = NULL, env_var_list) {
   writeLines(find_package, file.path(td, "CMakeLists.txt"))
   env_vars <- env_vars_as_string(env_var_list)
   cmake_cmd <- paste0(
-    "export ", env_vars,
-    " && cd ", td,
+    "export ",
+    env_vars,
+    " && cd ",
+    td,
     " && $CMAKE ",
     " -DCMAKE_EXPORT_NO_PACKAGE_REGISTRY=ON",
     " -DCMAKE_FIND_PACKAGE_NO_PACKAGE_REGISTRY=ON",
@@ -941,7 +950,7 @@ is_release <- is.na(dev_version) || dev_version < "100"
 
 on_macos <- tolower(Sys.info()[["sysname"]]) == "darwin"
 on_windows <- tolower(Sys.info()[["sysname"]]) == "windows"
-on_linux_dev <-  tolower(Sys.info()[["sysname"]]) == "linux" && grepl("devel", R.version.string)
+on_linux_dev <- tolower(Sys.info()[["sysname"]]) == "linux" && grepl("devel", R.version.string)
 
 # For local debugging, set ARROW_R_DEV=TRUE to make this script print more
 quietly <- !env_is("ARROW_R_DEV", "true")
@@ -975,10 +984,16 @@ if (is_release) {
   VERSION_MAJOR <- unlist(VERSION)[1]
   if (VERSION_MAJOR >= "22") {
     VERSION_22_OR_LATER <- TRUE
-    arrow_repo <- getOption("arrow.repo", sprintf("https://github.com/apache/arrow/releases/download/apache-arrow-%s/", VERSION))
+    arrow_repo <- getOption(
+      "arrow.repo",
+      sprintf("https://github.com/apache/arrow/releases/download/apache-arrow-%s/", VERSION)
+    )
   } else {
     VERSION_22_OR_LATER <- FALSE
-    arrow_repo <- paste0(getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)), "/libarrow/")
+    arrow_repo <- paste0(
+      getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)),
+      "/libarrow/"
+    )
   }
 } else {
   VERSION_22_OR_LATER <- TRUE
